@@ -1,17 +1,18 @@
 import unittest
 import bmemcached
+from bmemcached.compat import long, unicode
 
 
 class MemcachedTests(unittest.TestCase):
     def setUp(self):
         self.server = '127.0.0.1:11211'
         self.server = '/tmp/memcached.sock'
-        self.client = bmemcached.Client(self.server) #, 'user', 'password')
+        self.client = bmemcached.Client(self.server)
 
     def tearDown(self):
         self.reset()
         self.client.disconnect_all()
-        
+
     def reset(self):
         self.client.delete('test_key')
         self.client.delete('test_key2')
@@ -25,8 +26,8 @@ class MemcachedTests(unittest.TestCase):
             'test_key2': 'value2'}))
 
     def testSetMultiBigData(self):
-        self.client.set_multi(dict(
-                (unicode(k).encode(), b'value') for k in range(32767)))
+        self.client.set_multi(
+            dict((unicode(k), b'value') for k in range(32767)))
 
     def testGet(self):
         self.client.set('test_key', 'test')
@@ -122,9 +123,9 @@ class MemcachedTests(unittest.TestCase):
                          self.client.get_multi(['test_key', 'test_key2', 'nothere']))
 
     def testGetLong(self):
-        self.client.set('test_key', 1L)
+        self.client.set('test_key', long(1))
         value = self.client.get('test_key')
-        self.assertEqual(1L, value)
+        self.assertEqual(long(1), value)
         self.assertTrue(isinstance(value, long))
 
     def testGetInteger(self):
@@ -149,6 +150,13 @@ class MemcachedTests(unittest.TestCase):
         self.assertTrue(self.client.delete('test_key'))
         self.assertEqual(None, self.client.get('test_key'))
 
+    def testDeleteMulti(self):
+        self.client.set_multi({
+            'test_key': 'value',
+            'test_key2': 'value2'})
+
+        self.assertTrue(self.client.delete_multi(['test_key', 'test_key2']))
+
     def testDeleteUnknownKey(self):
         self.assertTrue(self.client.delete('test_key'))
 
@@ -187,9 +195,6 @@ class MemcachedTests(unittest.TestCase):
         stats = self.client.stats('settings')[self.server]
         self.assertTrue('verbosity' in stats)
 
-        stats = self.client.stats('slabs')[self.server]
-        self.assertTrue('1:get_hits' in stats)
-
     def testReconnect(self):
         self.client.set('test_key', 'test')
         self.client.disconnect_all()
